package com.example.filter;

import com.example.util.TokenUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


@Configuration
@Slf4j
public class AuthGlobalFilter implements WebMvcConfigurer {

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        InterceptorRegistration registration = registry.addInterceptor(new LoginInterceptors());
        registration.addPathPatterns("/**");                      //所有路径都被拦截
        registration.excludePathPatterns(                         //添加不拦截路径
                "/wechat/h5/authorize",
                "/alipay/**",
                "/wechat/h5/userInfo",
                "/app/getJsapiTicket",
                "/wechat/h5/code/{tel}",
                "/app/payResult",
                "/app/refundResult",
                "/**/*.html",            //html静态资源
                "/swagger-ui.html",
                "/null/swagger-resources/configuration/ui",
                "/swagger-resources/configuration/ui",
                "/swagger-resources/configuration/security",
                "/swagger-resources/*",
                "/swagger-resources",
                "/**/*.js",              //js静态资源
                "/**/*.css",             //css静态资源
                "/**/*.png",             //css静态资源
                "/**/*.woff",
                "/**/*.woff2",
                "/error",
                "/**/*.ttf"
        );
    }


    public class LoginInterceptors implements HandlerInterceptor {
        @Override
        public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
            String token = request.getHeader("Authorization");
            System.err.println("获取的token:"+token);
            String userId = TokenUtil.parseJWT(token).get("uid").toString();
//            LoginUser loginUser =  tokenService.getLoginUser(request);
            if(userId == null){
                log.error("用户未登录,请重新登录");
//                response.sendRedirect(request.getContextPath()+"http://wx.hw.hongweisoft.com/wxpaydemoh5");
                return false;
            }
            return true;
        }
    }
}


