package com.example.util;

import com.google.common.collect.Lists;
import org.dozer.DozerBeanMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;

import java.beans.PropertyDescriptor;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Java对象拷贝、转换工具类
 *
 * @author yys
 */
public final class BeanCopyUtil {

    private BeanCopyUtil() {
    }

    private static DozerBeanMapper dozer = new DozerBeanMapper();

    /**
     * 源对象转换为目标对象
     *
     * @param source           源对象
     * @param destinationClass 目标Class对象
     * @return 目标对象
     */
    public static <T> T convertBean(Object source, Class<T> destinationClass) {
        return dozer.map(source, destinationClass);
    }

    /**
     * 源List中对象转换为目标对象
     *
     * @param sourceList       源对象List
     * @param destinationClass 目标List中的Class对象
     * @return 转换后的List集合
     */
    public static <T> List<T> convertList(Collection<?> sourceList, Class<T> destinationClass) {
        List<T> destinationList = Lists.newArrayList();
        for (Object sourceObject : sourceList) {
            T destinationObject = dozer.map(sourceObject, destinationClass);
            destinationList.add(destinationObject);
        }
        return destinationList;
    }

    /**
     * 将源对象的属性复制目标对象
     *
     * @param source            源对象
     * @param destinationObject 目标对象
     */
    public static void copyBean(Object source, Object destinationObject) {
        dozer.map(source, destinationObject);
    }

    /**
     * 拷贝不为空的属性覆盖目标对象（常用于查询更新场景）
     *
     * @param src    源对象
     * @param target 目标对象
     */
    public static void copyNotNullProperties(Object src, Object target) {
        BeanWrapper srcBean = new BeanWrapperImpl(src);
        PropertyDescriptor[] pds = srcBean.getPropertyDescriptors();
        Set<String> emptyName = new HashSet<String>();
        for (PropertyDescriptor p : pds) {
            Object srcValue = srcBean.getPropertyValue(p.getName());
            if (srcValue == null) {
                emptyName.add(p.getName());
            }
        }
        String[] result = new String[emptyName.size()];
        BeanUtils.copyProperties(src, target, emptyName.toArray(result));
    }

}